unit pg;

(*
 * Delphi Sample Code provided by BITWISE MAGAZINE
 * 
 *      http://www.bitwisemag.com
 *
 * This code and information is provided "as is" without warranty of
 * any kind, either expressed or implied, including but not limited
 * to the implied warranties of merchantability and/or fitness for a
 * particular purpose.
 *
 * You may use and modify this code for your personal use only.
 * You may also redistribute this code providing that
 * a) No fee is charged and b) This copyright notice
 * is retained in the source code.
 *
 * You may not use this code for commercial purposes
 * without the express permission of the copyright holder
 *
 * Copyright (c) 2006 Huw Collingbourne 
 *
 *)
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, ShellCtrls,
  ShlObj, Menus,
  IniFiles,
  globals,
  childform,
  huwStrUtils;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Window1: TMenuItem;
    Tile1: TMenuItem;
    Cascade1: TMenuItem;
    Arrangeicons1: TMenuItem;
    N2: TMenuItem;
    New2: TMenuItem;
    ileVertical1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Tile1Click(Sender: TObject);
    procedure Cascade1Click(Sender: TObject);
    procedure Arrangeicons1Click(Sender: TObject);
    procedure NewClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TestClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure TileVertical1Click(Sender: TObject);
  private
    { Private declarations }
    paths : TStringList;
    viewStyles : TStringList;
    childpositions : TStringList;
    childform : Tchildfm;
    formjustcreated : boolean;
    procedure Init;
    function GetSpecialFolder(nFolder : integer) : string;
    function StrToViewStyle( s : String ) : TViewStyle;
    function ViewStyleToStr( vs : TViewStyle ) : string;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
implementation

//uses childform;

{$R *.dfm}
function TForm1.GetSpecialFolder(nFolder : integer) : string;
var
  pszPath : array[0..MAX_PATH] of char;
  ppidl : PItemIDList; // Needs ShlObj.pas
begin
  if (SHGetSpecialFolderLocation(0, nFolder, ppidl) ) = S_OK then
  begin
    SHGetPathFromIDList(ppidl, pszPath);
    Result := string(pszPath);
  end
  else Result := '';
end;

procedure TForm1.Init;
begin
  startmenudir := GetSpecialFolder(CSIDL_STARTMENU);
  if startmenudir <> '' then
    Caption := 'Start Menu= "'+startmenudir+'"';
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
   Close;
end;

procedure TForm1.Tile1Click(Sender: TObject);
begin
   TileMode := tbHorizontal;
   Tile;
end;

procedure TForm1.TileVertical1Click(Sender: TObject);
begin
   TileMode := tbVertical;
   Tile;
end;

procedure TForm1.Cascade1Click(Sender: TObject);
begin
   Cascade;
end;

procedure TForm1.Arrangeicons1Click(Sender: TObject);
begin
   ArrangeIcons;
end;

procedure TForm1.NewClick(Sender: TObject);
begin
   Tchildfm.Create(Self);
end;

procedure TForm1.TestClick(Sender: TObject);
var
  i : integer;
  s : string;
begin
  s := 'MDIChildCount='+IntToStr(MDIChildCount)+#10#13;
  for i := MDIChildCount-1 downto 0 do
    s := s + '['+IntToStr(i)+']' + MDIChildren[i].Caption + #10#13;
  ShowMessage(s);
end;


function TForm1.StrToViewStyle( s : String ) : TViewStyle;
begin
  s := uppercase(s);
  if s = 'VSICON' then
    Result := vsIcon
  else if s = 'VSSMALLICON' then
    result := vsSmallIcon
  else if s = 'VSLIST' then
    result := vsList
  else result := vsReport;
end;


function TForm1.ViewStyleToStr( vs : TViewStyle ) : string;
begin
  if vs = vsIcon then
    Result := 'vsIcon'
  else if vs = vsSmallIcon then
    result := 'vsSmallIcon'
  else if vs = vsList then
    result := 'vsList'
  else result := 'vsReport';
end;


procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var
  Ini : TIniFile;
  i : integer;
begin
  Ini := TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    Ini.WriteBool( 'Form', 'Maximized', WindowState = wsMaximized );
    WindowState := wsNormal;
    Ini.WriteInteger( 'Form', 'Top', Top);
    Ini.WriteInteger( 'Form', 'Left', Left);
    Ini.WriteInteger('Form','Height', Height);
    Ini.WriteInteger('Form','Width', Width);
    // !! Clear out any existing entries
    Ini.EraseSection('ProgGroups');
    Ini.EraseSection('ViewStyles');
    Ini.EraseSection('WinPositions');
    // !! Write data for each child form
    for i := MDIChildCount-1 downto 0 do
    begin
        Ini.WriteString('ProgGroups','Group'+IntToStr(i),MDIChildren[i].Caption);
        Ini.WriteString('ViewStyles','ViewStyle'+IntToStr(i), ViewStyleToStr(Tchildfm(MDIChildren[i]).ShellListView.ViewStyle));
        Ini.WriteString('WinPositions','Win'+IntToStr(i),Format('%d,%d,%d,%d',
                  [MDIChildren[i].Top,MDIChildren[i].Left,MDIChildren[i].Height,MDIChildren[i].Width]));
        // The following shows how to cast ViewStyle to integer. A technique which works but is inelegant
        // Ini.WriteInteger('ViewStyles','ViewStyle'+IntToStr(i), integer(Tchildfm(MDIChildren[i]).ShellListView.ViewStyle));
    end;
  finally
    Ini.Free;
  end;

end;


procedure TForm1.FormCreate(Sender: TObject);
var
  Ini: TIniFile;
  s : string;
begin
  Init;
  Ini := TIniFile.Create( ChangeFileExt( Application.ExeName, '.ini' ) );
  try
    Self.Top     := Ini.ReadInteger( 'Form', 'Top', 100 );
    Self.Left    := Ini.ReadInteger( 'Form', 'Left', 100 );
    Self.Height  := Ini.ReadInteger('Form','Height', 400);
    Self.Width   := Ini.ReadInteger('Form','Width', 600);
    if Ini.ReadBool( 'Form', 'Maximized', false ) then
      WindowState := wsMaximized
    else
      WindowState := wsNormal;
    //!! Read ProgGroups section into path StringList and parse out
    // the values
    paths := TStringList.Create;
    viewstyles := TStringList.Create;
    childpositions := TStringList.Create;
    Ini.ReadSectionValues('ProgGroups',paths);
    Ini.ReadSectionValues('ViewStyles',viewstyles);
    Ini.ReadSectionValues('WinPositions',childpositions);

  finally
    ini.Free;
  end;
  formjustcreated := true;
end;



procedure TForm1.FormShow(Sender: TObject);
// XXX NOTE: A great deal of error checking and recovery needs to be added here!
var
  i, T, L, H, W : integer;
  wp, first, rest : string;
begin
  if formjustcreated then
  begin

  // paths and viewstyles and chilpositions
    for i := 0 to paths.Count-1 do
    begin
        childform := Tchildfm.Create(Self);
        childform.SetRootFolder(paths.ValueFromIndex[i]);
        if i <= (viewstyles.Count-1) then
           childform.ShellListView.ViewStyle := StrToViewStyle(viewstyles.ValueFromIndex[i])
        else childform.ShellListView.ViewStyle := vsReport;
        if i <= (childpositions.Count-1) then
        begin
        // Error recovery should be added here!
          wp := Trim(childpositions.ValueFromindex[i]);
          firstrestStr(wp, first, rest);
          wp := rest;
          T := StrToInt(first);
          firstrestStr(wp, first, rest);
          L := StrToInt(first);
          wp := rest;
          firstrestStr(wp, first, rest);
          H := StrToInt(first);
          wp := rest;
          firstrestStr(wp, first, rest);
          W := StrToInt(first);
          childform.Top := T;
          childform.Left := L;
          childform.Height := H;
          childform.Width := W;
        end; //else defaults will be used
    end;

    viewstyles.Free;
    paths.Free;
    childpositions.Free;
    formjustcreated := false;
  end;
end;


end.
